@@ -226,7 +226,7 @@ alloc_coder (char const * const *inputname, const c_options_t *options,
           else
               file = pm_openr(filename);
           pnm_readpnminit(file, &width, &height, &maxval, &format);
-          color = (PNM_FORMAT_TYPE(format) == PPM_FORMAT);
+          color = (PNM_FORMAT_TYPE(format) == PPM_FORMAT) ? TRUE: FALSE;
 
           pm_close(file);
 	 if (n)
@@ -679,23 +679,25 @@ video_coder (char const * const *image_template, bitfile_t *output,
 static frame_type_e
 pattern2type (unsigned frame, const char *pattern)
 {
-   int tmp = toupper (pattern [frame % strlen (pattern)]);
+    int tmp = toupper (pattern [frame % strlen (pattern)]);
+    frame_type_e retval;
 
-   switch (tmp)
-   {
-      case 'I':
-	 return I_FRAME;
-	 break;
-      case 'B':
-	 return B_FRAME;
-	 break;
-      case 'P':
-	 return P_FRAME;
-	 break;
-      default:
-	 error ("Frame type %c not valid. Choose one of I,B or P.", tmp);
-   }
-   return NONE;
+    switch (tmp)
+    {
+    case 'I':
+        retval = I_FRAME;
+        break;
+    case 'B':
+        retval = B_FRAME;
+        break;
+    case 'P':
+        retval = P_FRAME;
+        break;
+    default:
+        error ("Frame type %c not valid. Choose one of I,B or P.", tmp);
+        retval = NONE;
+    }
+    return retval;
 }
 
 static void 
