@@ -16,10 +16,12 @@
 ** with lots of bits pasted from libpng.txt by Guy Eric Schalnat
 */
 
-#define VERSION "2.37.3 (13 July 1999) +netpbm"
-
-/* BJH 20000303:  fix include statement so dependencies work out right.
+/* 
+   BJH 20000408:  rename PPM_MAXMAXVAL to PPM_OVERALLMAXVAL
+   BJH 20000303:  fix include statement so dependencies work out right.
 */
+/* GRR 19991203:  moved VERSION to new version.h header file */
+
 /* GRR 19990713:  fixed redundant freeing of png_ptr and info_ptr in setjmp()
  *  blocks and added "pm_close(ifp)" in each.  */
 
@@ -30,12 +32,22 @@
 
 #include <math.h>
 #include <png.h>	/* includes zlib.h and setjmp.h */
+#define VERSION "2.37.4 (5 December 1999) +netpbm"
+
 #include "pnm.h"
 
 typedef struct _jmpbuf_wrapper {
   jmp_buf jmpbuf;
 } jmpbuf_wrapper;
 
+/* GRR 19991205:  this is used as a test for pre-1999 versions of netpbm and
+ *   pbmplus vs. 1999 or later (in which pm_close was split into two)
+ */
+#ifdef PBMPLUS_RAWBITS
+#  define pm_closer pm_close
+#  define pm_closew pm_close
+#endif
+
 #ifndef TRUE
 #  define TRUE 1
 #endif
@@ -266,31 +278,31 @@ FILE *tfp;
   type_string = alpha_string = "";
 
   if (fread (sig_buf, 1, SIG_CHECK_SIZE, ifp) != SIG_CHECK_SIZE) {
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("input file empty or too short");
   }
   if (png_sig_cmp (sig_buf, (png_size_t) 0, (png_size_t) SIG_CHECK_SIZE) != 0) {
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("input file not a PNG file");
   }
 
   png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING,
     &pngtopnm_jmpbuf_struct, pngtopnm_error_handler, NULL);
   if (png_ptr == NULL) {
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("cannot allocate LIBPNG structure");
   }
 
   info_ptr = png_create_info_struct (png_ptr);
   if (info_ptr == NULL) {
     png_destroy_read_struct (&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("cannot allocate LIBPNG structures");
   }
 
   if (setjmp (pngtopnm_jmpbuf_struct.jmpbuf)) {
     png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("setjmp returns error condition");
   }
 
@@ -335,18 +347,26 @@ FILE *tfp;
       strcpy (gamma_string, "");
     }
 
-    if (verbose)
-      pm_message ("reading a %d x %d image, %d bit%s %s%s%s%s",
+    if (verbose) {
+      pm_message ("reading a %dw x %dh image, %d bit%s %s%s%s%s",
 		  info_ptr->width, info_ptr->height,
 		  info_ptr->bit_depth, info_ptr->bit_depth > 1 ? "s" : "",
 		  type_string, alpha_string, gamma_string,
 		  info_ptr->interlace_type ? ", Adam7 interlaced" : "");
+      pm_message ("background {index, gray, red, green, blue} = "
+                  "{%d, %d, %d, %d, %d}",
+                  info_ptr->background.index,
+                  info_ptr->background.gray,
+                  info_ptr->background.red,
+                  info_ptr->background.green,
+                  info_ptr->background.blue);
+    }
   }
 
   png_image = (png_byte **)malloc (info_ptr->height * sizeof (png_byte*));
   if (png_image == NULL) {
     png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("couldn't allocate space for image");
   }
 
@@ -371,7 +391,7 @@ FILE *tfp;
         free (png_image[x]);
       free (png_image);
       png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
-      pm_close (ifp);
+      pm_closer (ifp);
       pm_error ("couldn't allocate space for image");
     }
   }
@@ -562,7 +582,7 @@ FILE *tfp;
       free (png_image[y]);
     free (png_image);
     png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
-    pm_close (ifp);
+    pm_closer (ifp);
     pm_error ("couldn't allocate space for image");
   }
 
@@ -609,10 +629,10 @@ FILE *tfp;
     }
   }
 
-  if ((pnm_type == PGM_TYPE) && (maxval > PGM_MAXMAXVAL))
-    maxmaxval = PGM_MAXMAXVAL;
-  else if ((pnm_type == PPM_TYPE) && (maxval > PPM_MAXMAXVAL))
-    maxmaxval = PPM_MAXMAXVAL;
+  if ((pnm_type == PGM_TYPE) && (maxval > PGM_OVERALLMAXVAL))
+    maxmaxval = PGM_OVERALLMAXVAL;
+  else if ((pnm_type == PPM_TYPE) && (maxval > PPM_OVERALLMAXVAL))
+    maxmaxval = PPM_OVERALLMAXVAL;
   else maxmaxval = maxval;
 
   if (verbose)
@@ -676,7 +696,7 @@ FILE *tfp;
             free (png_image[i]);
           free (png_image);
           png_destroy_read_struct (&png_ptr, &info_ptr, (png_infopp)NULL);
-          pm_close (ifp);
+          pm_closer (ifp);
           pm_error ("unknown PNG color type");
       }
       pnm_pixel++;
@@ -766,9 +786,9 @@ char *argv[];
   convertpng (ifp, tfp);
 
   if (text)
-    pm_close (tfp);
+    pm_closew (tfp);
 
-  pm_close (ifp);
-  pm_close (stdout);
+  pm_closer (ifp);
+  pm_closew (stdout);
   exit (0);
 }
