@@ -8,8 +8,11 @@
 ** copyright notice and this permission notice appear in supporting
 ** documentation.  This software is provided "as is" without express or
 ** implied warranty.
+**
+** Modified:
+**
 */
-
+ 
 #include <math.h>
 #include "pnm.h"
 
@@ -49,6 +52,7 @@ parse_command_line(int argc, char ** argv,
 
     unsigned int option_def_index;
     int xysize, xsize, ysize, pixels;
+    int reduce;
     float xscale, yscale, scale_parm;
 
     option_def_index = 0;   /* incremented by OPTENTRY */
@@ -61,6 +65,7 @@ parse_command_line(int argc, char ** argv,
     OPTENTRY(0,   "pixels",    OPT_UINT,    &pixels,        0);
     OPTENTRY(0,   "xysize",    OPT_FLAG,    &xysize,        0);
     OPTENTRY(0,   "verbose",   OPT_FLAG,    &cmdline_p->verbose,        0);
+    OPTENTRY(0,   "reduce",    OPT_UINT,    &reduce,        0);
 
     /* Set the defaults. -1 = unspecified */
     xsize = -1;
@@ -69,6 +74,8 @@ parse_command_line(int argc, char ** argv,
     yscale = -1.0;
     pixels = -1;
     xysize = 0;
+    reduce = -1;
+    cmdline_p->verbose = FALSE;
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
@@ -85,6 +92,8 @@ parse_command_line(int argc, char ** argv,
         pm_error("-xscale must be greater than zero.");
     if (yscale != -1.0 && yscale <= 0.0)
         pm_error("-yscale must be greater than zero.");
+    if (reduce <= 0 && reduce != -1)
+        pm_error("-reduce must be greater than zero.");
 
     if (xsize != -1 && xscale != -1)
         pm_error("Cannot specify both -xsize/width and -xscale.");
@@ -92,11 +101,16 @@ parse_command_line(int argc, char ** argv,
         pm_error("Cannot specify both -ysize/height and -yscale.");
     
     if (xysize && 
-        (xsize != -1 || xscale != -1 || ysize != -1 || yscale != -1) )
+        (xsize != -1 || xscale != -1 || ysize != -1 || yscale != -1 || 
+         reduce != -1 || pixels != -1) )
         pm_error("Cannot specify -xysize with other dimension options.");
     if (pixels != -1 && 
-        (xsize != -1 || xscale != -1 || ysize != -1 || yscale != -1) )
+        (xsize != -1 || xscale != -1 || ysize != -1 || yscale != -1 ||
+         reduce != -1) )
         pm_error("Cannot specify -pixels with other dimension options.");
+    if (reduce != -1 && 
+        (xsize != -1 || xscale != -1 || ysize != -1 || yscale != -1) )
+        pm_error("Cannot specify -reduce with other dimension options.");
 
     if (pixels == 0)
         pm_error("-pixels must be greater than zero");
@@ -126,7 +140,7 @@ parse_command_line(int argc, char ** argv,
         cmdline_p->ybox = 0;
         
         if (xsize == -1 && xscale == -1 && ysize == -1 && yscale == -1
-            && pixels == -1) {
+            && pixels == -1 && reduce == -1) {
             /* parameters are scale factor and optional filespec */
             if (argc-1 < 1)
                 pm_error("With no dimension options, you must supply at least "
@@ -147,11 +161,17 @@ parse_command_line(int argc, char ** argv,
             }
         } else {
             /* Only parameter allowed is optional filespec */
-            scale_parm = 0.0;
             if (argc-1 < 1)
                 cmdline_p->input_filespec = "-";
             else
                 cmdline_p->input_filespec = argv[1];
+
+            if (reduce != -1) {
+                scale_parm = ((double) 1.0) / ((double) reduce);
+                pm_message("reducing by %d gives scale factor of %f.", 
+                           reduce, scale_parm);
+            } else
+                scale_parm = 0.0;
         }
     }
 
@@ -346,16 +366,14 @@ horizontal_scale(const xel inputxelrow[], xel newxelrow[],
 int
 main(int argc, char **argv ) {
 
-struct cmdline_info cmdline;
+    struct cmdline_info cmdline;
     FILE* ifp;
     xel* xelrow;
     xel* tempxelrow;
     xel* newxelrow;
-    xel* xP;
-    xel* nxP;
     int rows, cols, format, newformat, rowsread, newrows, newcols;
     int row, col, needtoreadrow;
-    xelval maxval;
+    xelval maxval, newmaxval;
     float xscale, yscale;
     float fracrowtofill, fracrowleft;
     float* rs;
@@ -368,18 +386,17 @@ struct cmdline_info cmdline;
 
     ifp = pm_openr(cmdline.input_filespec);
 
-    pnm_pbmmaxval = PNM_MAXMAXVAL;  /* use larger value for better results */
     pnm_readpnminit( ifp, &cols, &rows, &maxval, &format );
 
     /* Promote PBM files to PGM. */
-    if ( PNM_FORMAT_TYPE(format) == PBM_TYPE )
-	{
+    if ( PNM_FORMAT_TYPE(format) == PBM_TYPE ) {
         newformat = PGM_TYPE;
-	pm_message( "promoting from PBM to PGM" );
-	}
-    else
+        newmaxval = PGM_MAXMAXVAL;
+        pm_message( "promoting from PBM to PGM" );
+	} else {
         newformat = format;
-
+        newmaxval = maxval;
+    }
     compute_output_dimensions(cmdline, rows, cols, &newrows, &newcols);
 
     /* We round the scale factor down so that we never fill up the 
@@ -415,35 +432,35 @@ struct cmdline_info cmdline;
     fracrowleft = yscale;
     needtoreadrow = 1;
     for ( col = 0; col < cols; ++col )
-	rs[col] = gs[col] = bs[col] = 0.0;
+        rs[col] = gs[col] = bs[col] = 0.0;
     fracrowtofill = 1.0;
 
-    pnm_writepnminit( stdout, newcols, newrows, maxval, newformat, 0 );
+    pnm_writepnminit( stdout, newcols, newrows, newmaxval, newformat, 0 );
     newxelrow = pnm_allocrow( newcols );
 
     for ( row = 0; row < newrows; ++row ) {
         /* First scale Y from xelrow into tempxelrow. */
         if ( newrows == rows ) { /* shortcut Y scaling if possible */
-            pnm_readpnmrow( ifp, xelrow, cols, maxval, format );
+            pnm_readpnmrow( ifp, xelrow, cols, newmaxval, format );
 	    } else {
             while ( fracrowleft < fracrowtofill ) {
                 if ( needtoreadrow )
                     if ( rowsread < rows ) {
-                        pnm_readpnmrow( ifp, xelrow, cols, maxval, format );
+                        pnm_readpnmrow( ifp, xelrow, cols, newmaxval, format );
                         ++rowsread;
                     }
                 switch ( PNM_FORMAT_TYPE(format) ) {
                 case PPM_TYPE:
-                    for ( col = 0, xP = xelrow; col < cols; ++col, ++xP ) {
-                        rs[col] += fracrowleft * PPM_GETR( *xP );
-                        gs[col] += fracrowleft * PPM_GETG( *xP );
-                        bs[col] += fracrowleft * PPM_GETB( *xP );
+                    for ( col = 0; col < cols; ++col ) {
+                        rs[col] += fracrowleft * PPM_GETR(xelrow[col]);
+                        gs[col] += fracrowleft * PPM_GETG(xelrow[col]);
+                        bs[col] += fracrowleft * PPM_GETB(xelrow[col]);
                     }
                     break;
 
                 default:
-                    for ( col = 0, xP = xelrow; col < cols; ++col, ++xP )
-                        gs[col] += fracrowleft * PNM_GET1( *xP );
+                    for ( col = 0; col < cols; ++col)
+                        gs[col] += fracrowleft * PNM_GET1(xelrow[col]);
                     break;
                 }
                 fracrowtofill -= fracrowleft;
@@ -453,7 +470,7 @@ struct cmdline_info cmdline;
             /* Now fracrowleft is >= fracrowtofill, so we can produce a row. */
             if ( needtoreadrow ) {
                 if ( rowsread < rows ) {
-                    pnm_readpnmrow( ifp, xelrow, cols, maxval, format );
+                    pnm_readpnmrow( ifp, xelrow, cols, newmaxval, format );
                     ++rowsread;
                     needtoreadrow = 0;
                 } else {
@@ -471,29 +488,28 @@ struct cmdline_info cmdline;
             }
             switch ( PNM_FORMAT_TYPE(format) ) {
             case PPM_TYPE:
-                for ( col = 0, xP = xelrow, nxP = tempxelrow;
-                      col < cols; ++col, ++xP, ++nxP ) {
+                for ( col = 0; col < cols; ++col) {
                     float r, g, b;
 
-                    r = rs[col] + fracrowtofill * PPM_GETR( *xP );
-                    g = gs[col] + fracrowtofill * PPM_GETG( *xP );
-                    b = bs[col] + fracrowtofill * PPM_GETB( *xP );
-                    PPM_ASSIGN( *nxP, 
-                        min(maxval, (int) (r + 0.5)), 
-                        min(maxval, (int) (g + 0.5)), 
-                        min(maxval, (int) (b + 0.5))
+                    r = rs[col] + fracrowtofill * PPM_GETR(xelrow[col]);
+                    g = gs[col] + fracrowtofill * PPM_GETG(xelrow[col]);
+                    b = bs[col] + fracrowtofill * PPM_GETB(xelrow[col]);
+                    PPM_ASSIGN( tempxelrow[col], 
+                        min(newmaxval, (int) (r + 0.5)), 
+                        min(newmaxval, (int) (g + 0.5)), 
+                        min(newmaxval, (int) (b + 0.5))
                         );
                     rs[col] = gs[col] = bs[col] = 0.0;
                 }
                 break;
 
             default:
-                for ( col = 0, xP = xelrow, nxP = tempxelrow;
-                      col < cols; ++col, ++xP, ++nxP ) {
+                for ( col = 0; col < cols; ++col ) {
                     float g;
                     
-                    g = gs[col] + fracrowtofill * PNM_GET1( *xP );
-                    PNM_ASSIGN1( *nxP, min(maxval, (int) (g + 0.5)));
+                    g = gs[col] + fracrowtofill * PNM_GET1(xelrow[col]);
+                    PNM_ASSIGN1(tempxelrow[col], 
+                                min(newmaxval, (int) (g + 0.5)));
                     gs[col] = 0.0;
                 }
                 break;
@@ -509,19 +525,21 @@ struct cmdline_info cmdline;
         /* Now scale tempxelrow horizontally into newxelrow & write it out. */
 
         if (newcols == cols)	/* shortcut X scaling if possible */
-            pnm_writepnmrow(stdout, tempxelrow, newcols, maxval, newformat, 0);
+            pnm_writepnmrow(stdout, tempxelrow, newcols, 
+                            newmaxval, newformat, 0);
         else {
             float stretch;
 
             horizontal_scale(tempxelrow, newxelrow, cols, newcols, xscale, 
-                             format, maxval, &stretch);
+                             format, newmaxval, &stretch);
             
             if (cmdline.verbose && row == 0)
                 pm_message("%f of right column stretched due to "
                            "arithmetic imprecision", 
                            stretch);
             
-            pnm_writepnmrow(stdout, newxelrow, newcols, maxval, newformat, 0 );
+            pnm_writepnmrow(stdout, newxelrow, newcols, 
+                            newmaxval, newformat, 0 );
         }
 	}
     
