@@ -45,9 +45,6 @@
 #endif
 #include <tiffioP.h>
 #endif
-#ifdef DJGPP
-#include <tiffioP.h>
-#endif
 #include <tiffio.h>
 
 /* The following are in current tiff.h, but so that we can compile against
@@ -66,20 +63,13 @@
 #define PHOTOMETRIC_DEPTH 32768
 #endif
 
-/* The following describe the structure of the raster buffer */
-static int maxval;
-static unsigned short fillorder;
-static unsigned short bps, spp;
-
-/* The NEXTSAMPLE macro effects the reading of a sample from the raster
-   buffer via the above cursor into the variable 'sample'.
-*/
 struct cmdline_info {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
     char *input_filename;
-    int headerdump;
+    unsigned int headerdump;
+    unsigned int respectfillorder;
     char *alpha_filename;
     int alpha_stdout;
 } cmdline;
@@ -88,17 +78,59 @@ struct cmdline_info {
 
 static void
 parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdline_p);
+                   struct cmdline_info *cmdline_p) {
+/*----------------------------------------------------------------------------
+   Note that many of the strings that this function returns in the
+   *cmdline_p structure are actually in the supplied argv array.  And
+   sometimes, one of these strings is actually just a suffix of an entry
+   in argv!
+-----------------------------------------------------------------------------*/
+    optStruct3 opt;
+    optEntry *option_def = malloc(100*sizeof(optEntry));
+    unsigned int option_def_index;
+    unsigned int alpha_spec;
+
+    opt.opt_table = option_def;
+    opt.short_allowed = FALSE;
+    opt.allowNegNum = FALSE;
+
+    option_def_index = 0;   /* incremented by OPTENTRY */
+    OPTENT3('h', "headerdump", 
+            OPT_FLAG,   NULL, &cmdline_p->headerdump, 0);
+    OPTENT3(0,   "alphaout",   
+            OPT_STRING, &cmdline_p->alpha_filename, &alpha_spec, 0);
+    OPTENT3('h', "respectfillorder", 
+            OPT_FLAG,   NULL, &cmdline_p->respectfillorder, 0);
+
+    optParseOptions3(&argc, argv, opt, sizeof(opt), 0);
+
+    if (argc - 1 == 0)
+        cmdline_p->input_filename = NULL;  /* he wants stdin */
+    else if (argc - 1 == 1)
+        cmdline_p->input_filename = strdup(argv[1]);
+    else 
+        pm_error("Too many arguments.  The only argument accepted\n"
+                 "is the input file specificaton");
+
+    if (alpha_spec && strcmp(cmdline_p->alpha_filename, "-") == 0 )
+        cmdline_p->alpha_stdout = 1;
+    else 
+        cmdline_p->alpha_stdout = 0;
+}
+
+
+
 
 static void read_directory(TIFF * const tif,
-                           unsigned short * const fillorder_p,
                            unsigned short * const bps_p,
                            unsigned short * const spp_p,
                            unsigned short * const photomet_p,
                            unsigned short * const planarconfig_p,
-                           unsigned int * const cols_p,
-                           unsigned int * const rows_p,
-                           const int headerdump) {
+                           unsigned short * const fillorder_p,
+                           unsigned int *   const cols_p,
+                           unsigned int *   const rows_p,
+                           bool             const headerdump,
+                           bool             const respectfillorder) {
 /*----------------------------------------------------------------------------
    Read various values of TIFF tags from the TIFF directory, and default 
    them if not in there.  Exit program with error message if required tags
@@ -120,17 +152,17 @@ static void read_directory(TIFF * const tif,
     if (*bps_p < 1 || (*bps_p > 8 && *bps_p != 16))
         pm_error("This program can process Tiff images with only "
                  "1-8 or 16 bits per sample.  The input Tiff image "
-                 "has %d bits per sample.", bps);
-
-    rc = TIFFGetField(tif, TIFFTAG_FILLORDER, &tiff_fillorder);
-    /* I don't know if it is legal for a Tiff file not to have a 
-       FILLORDER tag, but until I added 16 bit Tiff capability in November
-       2000, we never tried to read the tag.  I don't want to break this 
-       program for older image inputs by erroring out on a missing FILLORDER
-       tag here.  -Bryan
-       */
-    *fillorder_p = (rc == 0) ? FILLORDER_MSB2LSB : tiff_fillorder;
+                 "has %d bits per sample.", *bps_p);
 
+    rc = TIFFGetFieldDefaulted(tif, TIFFTAG_FILLORDER, &tiff_fillorder);
+    if (respectfillorder)
+        *fillorder_p = tiff_fillorder;
+    else {
+        *fillorder_p = FILLORDER_MSB2LSB;
+        if (tiff_fillorder != *fillorder_p)
+            pm_message("WARNING: Ignoring Tiff FILLORDER tag.  "
+                       "See -respectfillorder option for explanation.");
+    }
     rc = TIFFGetField(tif, TIFFTAG_SAMPLESPERPIXEL, &tiff_spp);
     *spp_p = (rc == 0) ? 1: tiff_spp;
 
@@ -152,34 +184,34 @@ static void read_directory(TIFF * const tif,
     }
 
     switch (*spp_p) {
-	case 1:
-	case 3:
-	case 4:
+    case 1:
+    case 3:
+    case 4:
         break;
-	default:
+    default:
         pm_error(
             "This program can only handle 1-channel gray scale "
             "or 1- or 3-channel color.  The input Tiff file has %d channels "
             "(samples per pixel)",
             *spp_p);
-	}
+    }
 
     switch(*planarconfig_p) {
-	case PLANARCONFIG_CONTIG:
-	    break;
-	case PLANARCONFIG_SEPARATE:
-	    if (*photomet_p != PHOTOMETRIC_RGB && 
+    case PLANARCONFIG_CONTIG:
+        break;
+    case PLANARCONFIG_SEPARATE:
+        if (*photomet_p != PHOTOMETRIC_RGB && 
             *photomet_p != PHOTOMETRIC_SEPARATED)
             pm_error("This program can handle separate planes only "
                      "with RGB (PHOTOMETRIC tag = %d) or SEPARATED "
                      "(PHOTOMETRIC tag = %d) data.  The input Tiff file " 
                      "has PHOTOMETRIC tag = %d.",
                      PHOTOMETRIC_RGB, PHOTOMETRIC_SEPARATED, &photomet_p);
-	    break;
-	default:
-	    pm_error("Unrecognized PLANARCONFIG tag value in Tiff input: %d.\n",
+        break;
+    default:
+        pm_error("Unrecognized PLANARCONFIG tag value in Tiff input: %d.\n",
                  *planarconfig_p);
-	}
+    }
 
     rc = TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, cols_p);
     if (rc == 0)
@@ -191,16 +223,17 @@ static void read_directory(TIFF * const tif,
     if (headerdump) {
         pm_message( "%dx%dx%d image", *cols_p, *rows_p, *bps_p * *spp_p );
         pm_message( "%d bits/sample, %d samples/pixel", *bps_p, *spp_p );
-	}
+    }
 }
 
 
 
 static void
 readscanline(TIFF * const tif, unsigned short * const samplebuf,
-             unsigned char scanbuf[], const int row, const int plane,
-             const unsigned int cols, const unsigned short bps, 
-             const unsigned short fillorder) {
+             unsigned char scanbuf[], int const row, int const plane,
+             unsigned int const cols, unsigned short const bps,
+             unsigned short const spp,
+             unsigned short const fillorder) {
 /*----------------------------------------------------------------------------
    Read one scanline out of the Tiff input and store it into samplebuf[].
    Unlike the scanline returned by the Tiff library function, samplebuf[]
@@ -214,6 +247,13 @@ readscanline(TIFF * const tif, unsigned short * const samplebuf,
     const unsigned int bpsmask = (1 << bps) - 1;
       /* A mask for taking the lowest 'bps' bits of a number */
 
+    /* The TIFFReadScanline man page doesn't tell the format of its
+       'buf' return value, but it is exactly the same format as the 'buf'
+       input to TIFFWriteScanline.  The man page for that doesn't say 
+       anything either, but the source code for Pnmtotiff contains a
+       specification.
+    */
+
     rc = TIFFReadScanline(tif, scanbuf, row, plane);
     if (rc < 0)
         pm_error( "Unable to read row %d, plane %d of input Tiff image.  "
@@ -225,7 +265,7 @@ readscanline(TIFF * const tif, unsigned short * const samplebuf,
             samplebuf[sample] = scanbuf[sample];
     } else if (bps < 8) {
         /* Note that in this format, samples do not span bytes.  Rather,
-           each byte may have don't-care bits in the low order positions.
+           each byte may have don't-care bits in the right-end positions.
            At least that's how I infer the format from reading pnmtotiff.c
            -Bryan 00.11.18
            */
@@ -233,46 +273,39 @@ readscanline(TIFF * const tif, unsigned short * const samplebuf,
         int bitsleft;
         unsigned char * inP;
 
-        bitsleft = 0;
-        inP = scanbuf;
-        for (sample = 0; sample < cols*spp; sample++) {
+        for (sample = 0, bitsleft=8, inP=scanbuf; 
+             sample < cols*spp; 
+             sample++) {
             if (bitsleft == 0) {
                 ++inP; 
                 bitsleft = 8;
             } 
+            switch (fillorder) {
+            case FILLORDER_MSB2LSB:
+                samplebuf[sample] = (*inP >> (bitsleft-bps)) & bpsmask; 
+                break;
+            case FILLORDER_LSB2MSB:
+                samplebuf[sample] = (*inP >> (8-bitsleft)) & bpsmask;
+                break;
+            default:
+                pm_error("invalid value for fillorder: %u", fillorder);
+            }
             bitsleft -= bps; 
-            samplebuf[sample] = (*inP >> bitsleft) & bpsmask; 
-        }
+            if (bitsleft < bps)
+                /* Don't count dregs at end of byte */
+                bitsleft = 0;
+       }
     } else if (bps == 16) {
-        /* Here's an idea to make this faster:  determine the endianness of
-           the machine we're running on, and if it's the same as that of the
-           input ('fillorder'), just do a memcpy().
-           */
-        switch (fillorder) {
-        case FILLORDER_MSB2LSB: {
-            int sample;
-            unsigned char * inP;
-            inP = scanbuf;  /* initial value */
-            for (sample = 0; sample < cols*spp; sample++) {
-                samplebuf[sample] = (*inP << 8) | *(inP+1);
-                inP += 2;
-            }
-        }
-        break;
-        case FILLORDER_LSB2MSB: {
-            int sample;
-            unsigned char * inP;
-            inP = scanbuf;  /* initial value */
-            for (sample = 0; sample < cols*spp; sample++) {
-                samplebuf[sample] = (*(inP+1) << 8) | *inP;
-                inP += 2;
-            }
-        }
-        break;
-        default:
-            pm_error("Internal error: invalid fillorder passed to "
-                     "readscanline()");
-        }
+        /* Before Netpbm 9.17, this program assumed that scanbuf[]
+           contained an array of bytes as read from the Tiff file.  In
+           fact, in this bps == 16 case, it's an array of "shorts",
+           each stored in whatever format this platform uses (which is
+           none of our concern).  The pre-9.17 code also presumed that
+           the TIFF "FILLORDER" tag determined the order in which the
+           bytes of each sample appear in a TIFF file, which is
+           contrary to the TIFF spec.  
+        */
+        memcpy((char *)&samplebuf[0], &scanbuf[0], cols*spp*2);
     } else
         pm_error("Internal error: invalid bits per sample passed to "
                  "readscanline()");
@@ -302,18 +335,155 @@ pick_cmyk_pixel(const unsigned short samplebuf[], const int sample_cursor,
 }
 
 
+
+static void
+analyze_image_type(TIFF * const tif, unsigned short const bps, 
+                   unsigned short const spp, unsigned short const photomet,
+                   xelval * const maxvalP, int * const formatP, 
+                   xel colormap[], bool const headerdump) {
+
+    bool grayscale; 
+
+    if ( bps == 1 && spp == 1 ) {
+        if ( cmdline.headerdump )
+            pm_message("monochrome" );
+        grayscale = TRUE;
+        *maxvalP = 1;
+    } else {
+        /* How come we don't deal with the photometric for the monochrome 
+           case (make sure it's one we know)?  -Bryan 00.03.04
+           */
+        switch ( photomet ) {
+        case PHOTOMETRIC_MINISBLACK:
+            grayscale = TRUE;
+            *maxvalP = (1 << bps) - 1;
+            if (headerdump)
+                pm_message( "%d graylevels (min=black)", *maxvalP + 1 );
+            break;
+            
+        case PHOTOMETRIC_MINISWHITE:
+            grayscale = TRUE;
+            *maxvalP = (1 << bps) - 1;
+            if (headerdump)
+                pm_message( "%d graylevels (min=white)", *maxvalP + 1 );
+            break;
+            
+        case PHOTOMETRIC_PALETTE: {
+            int i;
+            int numcolors;
+            unsigned short* redcolormap;
+            unsigned short* greencolormap;
+            unsigned short* bluecolormap;
+
+            if (headerdump)
+                pm_message("colormapped");
+
+            if (!TIFFGetField(tif, TIFFTAG_COLORMAP, 
+                              &redcolormap, &greencolormap, &bluecolormap))
+                pm_error("error getting colormaps");
+
+            numcolors = 1 << bps;
+            if (numcolors > MAXCOLORS)
+                pm_error("too many colors");
+            *maxvalP = PNM_MAXMAXVAL;
+            grayscale = FALSE;
+            for (i = 0; i < numcolors; ++i) {
+                xelval r, g, b;
+                r = (long) redcolormap[i] * PNM_MAXMAXVAL / 65535L;
+                g = (long) greencolormap[i] * PNM_MAXMAXVAL / 65535L;
+                b = (long) bluecolormap[i] * PNM_MAXMAXVAL / 65535L;
+                PPM_ASSIGN(colormap[i], r, g, b);
+            }
+        }
+        break;
+
+        case PHOTOMETRIC_SEPARATED: {
+            unsigned short inkset;
+
+            if (headerdump)
+                pm_message("color separation");
+            if (TIFFGetField(tif, TIFFTAG_INKNAMES, &inkset) == 1
+                && inkset != INKSET_CMYK)
+            if (inkset != INKSET_CMYK) 
+                pm_error("This color separation file uses an inkset (%d) ",
+                         "we can't handle.  We handle only CMYK.", inkset);
+            if (spp != 4) 
+                pm_error("This CMYK color separation file is %d samples per "
+                         "pixel.  "
+                         "We need 4 samples, though: C, M, Y, and K.  ",
+                         spp);
+            grayscale = FALSE;
+            *maxvalP = (1 << bps) - 1;
+        }
+        break;
+            
+        case PHOTOMETRIC_RGB:
+            if (headerdump)
+                pm_message("RGB truecolor");
+            grayscale = FALSE;
+            *maxvalP = (1 << bps) - 1;
+            break;
+
+        case PHOTOMETRIC_MASK:
+            pm_error("don't know how to handle PHOTOMETRIC_MASK");
+
+        case PHOTOMETRIC_DEPTH:
+            pm_error("don't know how to handle PHOTOMETRIC_DEPTH");
+
+        case PHOTOMETRIC_YCBCR:
+            pm_error("don't know how to handle PHOTOMETRIC_YCBCR");
+
+        case PHOTOMETRIC_CIELAB:
+            pm_error("don't know how to handle PHOTOMETRIC_CIELAB");
+
+        case PHOTOMETRIC_LOGL:
+            pm_error("don't know how to handle PHOTOMETRIC_LOGL");
+
+        case PHOTOMETRIC_LOGLUV:
+            pm_error("don't know how to handle PHOTOMETRIC_LOGLUV");
+            
+        default:
+            pm_error("unknown photometric: %d", photomet);
+        }
+    }
+    if (*maxvalP > PNM_OVERALLMAXVAL)
+        pm_error("bits/sample (%d) in the input image is too large.",
+                 bps);
+    if (grayscale) {
+        if (*maxvalP == 1) {
+            *formatP = PBM_TYPE;
+            pm_message("writing PBM file");
+        } else {
+            *formatP = PGM_TYPE;
+            pm_message("writing PGM file");
+        }
+    } else {
+        *formatP = PPM_TYPE;
+        pm_message("writing PPM file");
+    }
+}
+
+
+
 int
 main(int argc, char * argv[]) {
 
     unsigned int cols, rows;
-    int grayscale, format;
-    int numcolors;
-    register TIFF* tif;
+    int format;
+    TIFF* tif;
     FILE *alpha_file, *imageout_file;
-    int row, i;
-    register int col;
+    int row;
     unsigned char * scanbuf;
+        /* Buffer for a raster line in the format returned by TIFF library's
+           TIFFReadScanline
+        */
     unsigned short * samplebuf;
+        /* Same info as 'scanbuf' above, but with each raster column (sample)
+           represented as single array element, so it's easy to work with.
+        */
+    xelval maxval;
+    unsigned short bps, spp;
+
     xel* xelrow;
         /* The ppm-format row of the image row we are presently converting */
     gray* alpharow;
@@ -321,11 +491,7 @@ main(int argc, char * argv[]) {
            row we are presently converting.
            */
     xel colormap[MAXCOLORS];
-    unsigned char sample;
-    unsigned short photomet, planarconfig, inkset;
-    unsigned short* redcolormap;
-    unsigned short* greencolormap;
-    unsigned short* bluecolormap;
+    unsigned short photomet, planarconfig, fillorder;
 
     pnm_init( &argc, argv );
 
@@ -335,11 +501,11 @@ main(int argc, char * argv[]) {
         tif = TIFFOpen( cmdline.input_filename, "r" );
         if ( tif == NULL )
             pm_error( "error opening TIFF file %s", cmdline.input_filename );
-	} else {
+    } else {
         tif = TIFFFdOpen( 0, "Standard Input", "r" );
         if ( tif == NULL )
             pm_error( "error opening standard input as TIFF file" );
-	}
+    }
 
     if (cmdline.alpha_stdout)
         alpha_file = stdout;
@@ -354,120 +520,12 @@ main(int argc, char * argv[]) {
     else
         imageout_file = stdout;
 
-    read_directory(tif, &fillorder, &bps, &spp, &photomet, &planarconfig, 
+    read_directory(tif, &bps, &spp, &photomet, &planarconfig, &fillorder,
                    &cols, &rows, 
-                   cmdline.headerdump);
+                   cmdline.headerdump, cmdline.respectfillorder);
 
-    if ( bps == 1 && spp == 1 ) {
-        if ( cmdline.headerdump )
-            pm_message("monochrome" );
-        grayscale = 1;
-        maxval = 1;
-	} else {
-        /* How come we don't deal with the photometric for the monochrome 
-           case (make sure it's one we know)?  -Bryan 00.03.04
-           */
-        switch ( photomet ) {
-          case PHOTOMETRIC_MINISBLACK:
-            if ( cmdline.headerdump )
-              pm_message( "%d graylevels (min=black)", maxval + 1 );
-            grayscale = 1;
-            maxval = (1 << bps) - 1;
-            break;
-
-          case PHOTOMETRIC_MINISWHITE:
-            if ( cmdline.headerdump )
-              pm_message( "%d graylevels (min=white)", maxval + 1 );
-            grayscale = 1;
-            maxval = (1 << bps) - 1;
-            break;
-
-          case PHOTOMETRIC_PALETTE:
-            if ( cmdline.headerdump )
-              pm_message( "colormapped" );
-            if ( ! TIFFGetField( tif, TIFFTAG_COLORMAP, 
-                                &redcolormap, &greencolormap, &bluecolormap ) )
-              pm_error( "error getting colormaps" );
-            numcolors = 1 << bps;
-            if ( numcolors > MAXCOLORS )
-                pm_error( "too many colors" );
-            maxval = PNM_MAXMAXVAL;
-            grayscale = 0;
-            for ( i = 0; i < numcolors; ++i ) {
-                register xelval r, g, b;
-                r = (long) redcolormap[i] * PNM_MAXMAXVAL / 65535L;
-                g = (long) greencolormap[i] * PNM_MAXMAXVAL / 65535L;
-                b = (long) bluecolormap[i] * PNM_MAXMAXVAL / 65535L;
-                PPM_ASSIGN( colormap[i], r, g, b );
-            }
-            break;
-
-          case PHOTOMETRIC_SEPARATED:
-            if ( cmdline.headerdump )
-              pm_message( "color separation" );
-            if ( TIFFGetField( tif, TIFFTAG_INKNAMES, &inkset ) == 1
-                && inkset != INKSET_CMYK)
-            if (inkset != INKSET_CMYK) 
-               pm_error("This color separation file uses an inkset (%d) ",
-                        "we can't handle.  We handle only CMYK.", inkset);
-            if (spp != 4) 
-              pm_error("This CMYK color separation file is %d samples per "
-                       "pixel.  We need 4 samples, though: C, M, Y, and K.  ",
-                       spp);
-            grayscale = 0;
-            maxval = (1 << bps) - 1;
-            break;
-            
-          case PHOTOMETRIC_RGB:
-            if ( cmdline.headerdump )
-              pm_message( "RGB truecolor" );
-            grayscale = 0;
-            maxval = (1 << bps) - 1;
-            break;
-
-          case PHOTOMETRIC_MASK:
-            pm_error( "don't know how to handle PHOTOMETRIC_MASK" );
-
-          case PHOTOMETRIC_DEPTH:
-            pm_error( "don't know how to handle PHOTOMETRIC_DEPTH" );
-
-          case PHOTOMETRIC_YCBCR:
-            pm_error( "don't know how to handle PHOTOMETRIC_YCBCR" );
-
-          case PHOTOMETRIC_CIELAB:
-            pm_error( "don't know how to handle PHOTOMETRIC_CIELAB" );
-
-          case PHOTOMETRIC_LOGL:
-            pm_error( "don't know how to handle PHOTOMETRIC_LOGL" );
-
-          case PHOTOMETRIC_LOGLUV:
-            pm_error( "don't know how to handle PHOTOMETRIC_LOGLUV" );
-
-          default:
-            pm_error( "unknown photometric: %d", photomet );
-	    }
-	}
-    if ( maxval > PNM_OVERALLMAXVAL )
-        pm_error("bits/sample (%d) in the input image is too large.\n",
-                 bps);
-    if ( grayscale )
-	{
-	if ( maxval == 1 )
-	    {
-	    format = PBM_TYPE;
-	    pm_message( "writing PBM file" );
-	    }
-	else
-	    {
-	    format = PGM_TYPE;
-	    pm_message( "writing PGM file" );
-	    }
-	}
-    else
-	{
-	format = PPM_TYPE;
-	pm_message( "writing PPM file" );
-	}
+    analyze_image_type(tif, bps, spp, photomet, 
+                       &maxval, &format, colormap, cmdline.headerdump);
 
     scanbuf = (unsigned char *) malloc(TIFFScanlineSize(tif));
     if (scanbuf == NULL)
@@ -488,26 +546,30 @@ main(int argc, char * argv[]) {
 
     for ( row = 0; row < rows; ++row ) {
         /* Read one row of samples into samplebuf[] */
-        readscanline(tif, samplebuf, scanbuf, row, 0, 
-                     cols, bps, fillorder);
+        readscanline(tif, samplebuf, scanbuf, row, 0, cols, bps, spp, 
+                     fillorder);
 
         switch ( photomet ) {
-          case PHOTOMETRIC_MINISBLACK:
+        case PHOTOMETRIC_MINISBLACK: {
+            int col;
             for ( col = 0; col < cols; ++col ) {
                 PNM_ASSIGN1( xelrow[col], samplebuf[col] );
                 alpharow[col] = 0;
             }
+        }
             break;
 
-          case PHOTOMETRIC_MINISWHITE:
+        case PHOTOMETRIC_MINISWHITE: {
+            int col;
             for ( col = 0; col < cols; ++col ) {
-                sample = maxval - sample;
-                PNM_ASSIGN1( xelrow[col], samplebuf[col] );
+                PNM_ASSIGN1( xelrow[col], maxval - samplebuf[col] );
                 alpharow[col] = 0;
             }
-            break;
+        }
+        break;
 
-          case PHOTOMETRIC_PALETTE:
+        case PHOTOMETRIC_PALETTE: {
+            int col;
             for ( col = 0; col < cols; ++col ) {
                 /* We know the following array index is in bounds because
                    we filled samplebuf with samples of 'bps' bits each and
@@ -517,9 +579,10 @@ main(int argc, char * argv[]) {
                 xelrow[col] = colormap[samplebuf[col]];
                 alpharow[col] = 0;
             }
-            break;
+        }
+        break;
 
-          case PHOTOMETRIC_SEPARATED:
+        case PHOTOMETRIC_SEPARATED:
             if (planarconfig == PLANARCONFIG_CONTIG) {
                 int col, sample;
                 for ( col = 0, sample = 0; col < cols; ++col, sample+=spp) {
@@ -559,20 +622,20 @@ main(int argc, char * argv[]) {
                 
                 /* Next the greens */
                 readscanline( tif, samplebuf, scanbuf, row, 1,
-                             cols, bps, fillorder );
+                             cols, bps, spp, fillorder );
                 for ( col = 0; col < cols; ++col ) 
                     PPM_PUTG( xelrow[col], samplebuf[col] );
 
                 /* And finally the blues */
                 readscanline( tif, samplebuf, scanbuf, row, 2,
-                             cols, bps, fillorder );
+                              cols, bps, spp, fillorder );
                 for ( col = 0; col < cols; ++col ) 
                     PPM_PUTB( xelrow[col], samplebuf[col] );
                 /* Could there be an alpha plane?  (We assume no.  But if so,
                    here is where to read it) */
                 for ( col = 0; col < cols; ++col) 
                     alpharow[col] = 0;
-            }		
+            }       
             break;
             
           default:
@@ -595,47 +658,4 @@ main(int argc, char * argv[]) {
        code, via various function calls.
     */
     return 0;
-    }
-
-
-
-static void
-parse_command_line(int argc, char ** argv,
-                   struct cmdline_info *cmdline_p) {
-/*----------------------------------------------------------------------------
-   Note that many of the strings that this function returns in the
-   *cmdline_p structure are actually in the supplied argv array.  And
-   sometimes, one of these strings is actually just a suffix of an entry
-   in argv!
------------------------------------------------------------------------------*/
-    optStruct *option_def = malloc(100*sizeof(optStruct));
-        /* Instructions to OptParseOptions on how to parse our options.
-         */
-    unsigned int option_def_index;
-
-    option_def_index = 0;   /* incremented by OPTENTRY */
-    OPTENTRY('h', "headerdump", OPT_FLAG,   &cmdline_p->headerdump,     0);
-    OPTENTRY(0,   "alphaout",   OPT_STRING, &cmdline_p->alpha_filename, 0);
-
-    /* Set the defaults */
-    cmdline_p->headerdump = 0;
-    cmdline_p->alpha_filename = NULL;
-
-    optParseOptions(&argc, argv, option_def, 0);
-        /* Uses and sets argc, argv, and all of *cmdline_p. */
-
-    if (argc - 1 == 0)
-        cmdline_p->input_filename = NULL;  /* he wants stdin */
-    else if (argc - 1 == 1)
-        cmdline_p->input_filename = strdup(argv[1]);
-    else 
-        pm_error("Too many arguments.  The only argument accepted\n"
-                 "is the input file specificaton");
-
-    if (cmdline_p->alpha_filename && 
-        strcmp(cmdline_p->alpha_filename, "-") == 0 )
-        cmdline_p->alpha_stdout = 1;
-    else 
-        cmdline_p->alpha_stdout = 0;
 }
-
