@@ -185,33 +185,18 @@ static void
 PPMtoYUV(pixel ** const ppm_image, const int width, const int height,
          uint8 *** const orig_yP, 
          uint8 *** const orig_crP, 
-         uint8 *** const orig_cbP)
-{
+         uint8 *** const orig_cbP) {
+
     int y;
     uint8 ** orig_y;
     uint8 ** orig_cr;
     uint8 ** orig_cb;
 
-    *orig_yP = (uint8 **) malloc(sizeof(uint8 *) * height);
     orig_y = *orig_yP;
-    for (y = 0; y < height; y++) {
-        orig_y[y] = (uint8 *) malloc(sizeof(uint8) * width);
-    }
-
-    *orig_crP = (uint8 **) malloc(sizeof(uint8 *) * height / 2);
     orig_cr = *orig_crP;
-    for (y = 0; y < height / 2; y++) {
-        orig_cr[y] = (uint8 *) malloc(sizeof(uint8) * width / 2);
-    }
-
-    *orig_cbP = (uint8 **) malloc(sizeof(uint8 *) * height / 2);
     orig_cb = *orig_cbP;
-    for (y = 0; y < height / 2; y++) {
-        orig_cb[y] = (uint8 *) malloc(sizeof(uint8) * width / 2);
-    }
 
-    for (y = 0; y + 1 < height; y += 2)
-    {
+    for (y = 0; y + 1 < height; y += 2) {
         uint8 *dy0, *dy1;
         uint8 *dcr, *dcb;
         const pixel *src0, *src1;
@@ -269,6 +254,35 @@ WriteYUV(FILE *fpointer, const int width, const int height,
         fwrite(orig_cr[y], 1, width / 2, fpointer);
 }
 
+static void
+AllocYUV(const int width, const int height,
+         uint8 *** const orig_yP, 
+         uint8 *** const orig_crP,
+         uint8 *** const orig_cbP) {
+
+    int y;
+    uint8 ** orig_y;
+    uint8 ** orig_cr;
+    uint8 ** orig_cb;
+
+    *orig_yP = (uint8 **) malloc(sizeof(uint8 *) * height);
+    orig_y = *orig_yP;
+    for (y = 0; y < height; y++) {
+        orig_y[y] = (uint8 *) malloc(sizeof(uint8) * width);
+    }
+
+    *orig_crP = (uint8 **) malloc(sizeof(uint8 *) * height / 2);
+    orig_cr = *orig_crP;
+    for (y = 0; y < height / 2; y++) {
+        orig_cr[y] = (uint8 *) malloc(sizeof(uint8) * width / 2);
+    }
+
+    *orig_cbP = (uint8 **) malloc(sizeof(uint8 *) * height / 2);
+    orig_cb = *orig_cbP;
+    for (y = 0; y < height / 2; y++) {
+        orig_cb[y] = (uint8 *) malloc(sizeof(uint8) * width / 2);
+    }
+}
 
 static void
 FreeYUV(const int width, const int height,
@@ -276,26 +290,29 @@ FreeYUV(const int width, const int height,
 
     int y;
 
-    for (y = 0; y < height; y++) {
-        free(orig_y[y]);
+    if (orig_y) {
+       for (y = 0; y < height; y++)
+           free(orig_y[y]);
+       free(orig_y);
     }
-    free(orig_y);
 
-    for (y = 0; y < height / 2; y++) {
-        free(orig_cr[y]);
+    if (orig_cr) {
+       for (y = 0; y < height / 2; y++)
+           free(orig_cr[y]);
+       free(orig_cr);
     }
-    free(orig_cr);
 
-    for (y = 0; y < height / 2; y++) {
-        free(orig_cb[y]);
+    if (orig_cb) {
+       for (y = 0; y < height / 2; y++)
+           free(orig_cb[y]);
+       free(orig_cb);
     }
-    free(orig_cr);
 }
 
 
+
 int
-main(int argc, char **argv)
-{
+main(int argc, char **argv) {
     const char *input_filename;  /* NULL for stdin */
     FILE * ifp;
     int width, height;
@@ -307,10 +324,13 @@ main(int argc, char **argv)
            red and blue chrominance values over each 4 pixel square.
         */
     int eof;
+
+    /* The following are width, height, and maxval of the image we 
+       processed before this one.  Zero if there was no image before 
+       this one.
+    */
+    int last_width, last_height;
     pixval last_maxval;
-        /* The maxval of the image we processed before this one.  Zero
-           if there was no image before this one.
-           */
 
     ppm_init(&argc, argv);
 
@@ -323,10 +343,14 @@ main(int argc, char **argv)
     else input_filename = NULL;
 
     if (input_filename == NULL) ifp = stdin;
-    else ifp = pm_openr((char *)input_filename);
+    else ifp = pm_openr(input_filename);
 
     eof = FALSE;
     last_maxval = 0;  /* No previous maxval */
+    last_width = 0;	/* No previous width */
+    last_height = 0;	/* No previous height */
+    orig_y = orig_cr = orig_cb = 0;
+
     while (!eof) {
         ppm_image = ppm_readppm(ifp, &width, &height, &maxval);
 
@@ -344,16 +368,25 @@ main(int argc, char **argv)
         }
         last_maxval = maxval;
         
+        if (height != last_height || width != last_width) {
+            FreeYUV(width, height, orig_y, orig_cr, orig_cb);
+            /* Need new YUV buffers for different size */
+            AllocYUV(width, height, &orig_y, &orig_cr, &orig_cb);
+        }
+        last_height = height;
+        last_width = width;
+
         PPMtoYUV(ppm_image, width, height, &orig_y, &orig_cr, &orig_cb);
 
         WriteYUV(stdout, (width/2)*2, (height/2)*2, orig_y, orig_cr, orig_cb);
 
-        FreeYUV(width, height, orig_y, orig_cr, orig_cb);
         ppm_freearray(ppm_image, height);
         ppm_nextimage(ifp, &eof);
     }
+    FreeYUV(width, height, orig_y, orig_cr, orig_cb);
     free_multiplication_tables();
     pm_close(ifp);
         
     return 0;
 }
+
