@@ -23,12 +23,7 @@
 #include "ppm.h"
 #include "ppmcmap.h"
 
-#define MAXCOLORS 256
-
-/*
- * Pointer to function returning an int
- */
-typedef int (* ifunptr) ARGS((int, int));
+#define MAXCMAPSIZE 256
 
 /*
  * a code_int must be able to hold 2**BITS values of type int, and also -1
@@ -42,15 +37,37 @@ typedef unsigned short int count_short;
 typedef long int          count_int;
 #endif /*SIGNED_COMPARE_SLOW*/
 
-static int colorstobpp ARGS(( int colors ));
-static int GetPixel ARGS(( int x, int y ));
+struct cmap {
+    /* This is the information for the GIF colormap (aka palette). */
+
+    int red[MAXCMAPSIZE], green[MAXCMAPSIZE], blue[MAXCMAPSIZE];
+        /* These arrays arrays map a color index, as is found in
+           the raster part of the GIF, to an intensity value for the indicated
+           RGB component.
+        */
+    int perm[MAXCMAPSIZE], permi[MAXCMAPSIZE];
+        /* perm[i] is the position in the sorted colormap of the color which
+           is at position i in the unsorted colormap.  permi[] is the inverse
+           function of perm[].
+        */
+    int cmapsize;
+        /* Number of entries in the GIF colormap.  I.e. number of colors
+           in the image, plus possibly one fake transparency color.
+        */
+    int transparent;
+        /* color index number in GIF palette of the color that is to be
+           transparent.  -1 if no color is transparent.
+        */
+    colorhash_table cht;
+        /* A hash table that relates a PPM pixel value to to a pre-sort
+           GIF colormap index.
+        */
+    pixval maxval;
+        /* The maxval for the colors in 'cht'. */
+};
+
 static void BumpPixel ARGS(( void ));
-static int GIFNextPixel ARGS(( ifunptr getpixel ));
-int Red[MAXCOLORS],Green[MAXCOLORS],Blue[MAXCOLORS],perm[MAXCOLORS],permi[MAXCOLORS];
-int colors;
-pixval maxtmp;
 static void Putword ARGS(( int w, FILE* fp ));
-static void compress ARGS(( int init_bits, FILE* outfile, ifunptr ReadValue ));
 static void output ARGS(( code_int code ));
 static void cl_block ARGS(( void ));
 static void cl_hash ARGS(( count_int hsize ));
@@ -58,22 +75,19 @@ static void writeerr ARGS(( void ));
 static void char_init ARGS(( void ));
 static void char_out ARGS(( int c ));
 static void flush_char ARGS(( void ));
-static int sqr ARGS((int x));
-static int closestcolor ARGS((pixel color));
-
-static pixel** pixels;
-static colorhash_table cht;
 
 struct cmdline_info {
     /* All the information the user supplied in the command line,
        in a form easy for the program to use.
     */
-    char *input_filespec;  /* Filespecs of input files */
-    int interlace;
-    int sort;
-    char *mapfile;
-    char *transparent;  
-    char *comment;
+    char *input_filespec;  /* Filespec of input file */
+    char *alpha_filespec;  /* Filespec of alpha file; NULL if none */
+    char *alphacolor;      /* -alphacolor option value or default */
+    unsigned int interlace; /* -interlace option value */
+    unsigned int sort;     /* -sort option value */
+    char *mapfile;         /* -mapfile option value.  NULL if none. */
+    char *transparent;     /* -transparent option value.  NULL if none. */
+    char *comment;         /* -comment option value; NULL if none */
 };
 
 
@@ -97,6 +111,8 @@ parse_command_line(int argc, char ** argv,
     OPTENTRY(0,   "mapfile",     OPT_STRING, &cmdline_p->mapfile,        0);
     OPTENTRY(0,   "transparent", OPT_STRING, &cmdline_p->transparent,    0);
     OPTENTRY(0,   "comment",     OPT_STRING, &cmdline_p->comment,        0);
+    OPTENTRY(0,   "alpha",       OPT_STRING, &cmdline_p->alpha_filespec, 0);
+    OPTENTRY(0,   "alphacolor",  OPT_STRING, &cmdline_p->alphacolor,     0);
 
     /* Set the defaults */
     cmdline_p->interlace = FALSE;
@@ -104,6 +120,11 @@ parse_command_line(int argc, char ** argv,
     cmdline_p->mapfile = NULL;
     cmdline_p->transparent = NULL;  /* no transparency */
     cmdline_p->comment = NULL;      /* no comment */
+    cmdline_p->alpha_filespec = NULL;      /* no alpha file */
+    cmdline_p->alphacolor = "rgb:0/0/0";      
+        /* We could say "black" here, but then we depend on the color names
+           database existing.
+        */
 
     opt.opt_table = option_def;
     opt.short_allowed = FALSE;  /* We have no short (old-fashioned) options */
@@ -120,82 +141,74 @@ parse_command_line(int argc, char ** argv,
     else
         cmdline_p->input_filespec = argv[1];
 
+    if (cmdline_p->alpha_filespec && cmdline_p->transparent)
+        pm_error("You cannot specify both -alpha and -transparent.");
 }
 
 
 
-static int
-colorstobpp( colors )
-int colors;
-    {
-    int bpp;
+static int __inline__ sqr(const int x) {
+    return x*x;
+}
 
-    if ( colors <= 2 )
-        bpp = 1;
-    else if ( colors <= 4 )
-        bpp = 2;
-    else if ( colors <= 8 )
-        bpp = 3;
-    else if ( colors <= 16 )
-        bpp = 4;
-    else if ( colors <= 32 )
-        bpp = 5;
-    else if ( colors <= 64 )
-        bpp = 6;
-    else if ( colors <= 128 )
-        bpp = 7;
-    else if ( colors <= 256 )
-        bpp = 8;
-    else
-        pm_error( "can't happen" );
 
-    return bpp;
+static int __inline__
+closestcolor(const pixel color, pixval const maxval, struct cmap * cmapP) {
+/*----------------------------------------------------------------------------
+   Return the pre-sort colormap index of the color in the colormap *cmapP
+   that is closest to the color 'color', whose maxval is 'maxval'.
+
+   Also add 'color' to the colormap hash, with the colormap index we
+   are returning.  Caller must ensure that the color is not already in
+   there.
+-----------------------------------------------------------------------------*/
+    int i,r,g,b;
+    int imin,dmin;
+
+    r=(int)PPM_GETR(color)*255/maxval;
+    g=(int)PPM_GETG(color)*255/maxval;
+    b=(int)PPM_GETB(color)*255/maxval;
+
+    dmin = sqr(255)*3;
+    imin = 0;
+    for (i=0;i < cmapP->cmapsize; i++) {
+        int d;
+        d=sqr(r-cmapP->red[i])+sqr(g-cmapP->green[i])+sqr(b-cmapP->blue[i]);
+        if (d<dmin) {
+            dmin=d;
+            imin=i; 
+        } 
     }
+    ppm_addtocolorhash(cmapP->cht,&color,cmapP->permi[imin]);
+    return cmapP->permi[imin];
+}
 
 
-static int
-sqr(x)
-int x;
-  {
-  return x*x;
-  }
 
+static __inline__ int
+GetPixel(pixel ** const pixels, pixval const input_maxval,
+         gray ** const alpha, gray const alpha_threshold, 
+         struct cmap * const cmapP,
+         int const x, int const y) {
+/*----------------------------------------------------------------------------
+   Return the colormap index of the pixel at location (x,y) in the PPM
+   raster 'pixels', using colormap *cmapP.
+-----------------------------------------------------------------------------*/
+    int colorindex;
 
-static int
-closestcolor(color)
-pixel color;
-  {
-  int i,r,g,b,d,
-      imin,dmin;
+    if (alpha && alpha[y][x] < alpha_threshold)
+        colorindex = cmapP->transparent;
+    else {
+        int presort_colorindex;
 
-  r=(int)PPM_GETR(color)*255/maxtmp;
-  g=(int)PPM_GETG(color)*255/maxtmp;
-  b=(int)PPM_GETB(color)*255/maxtmp;
-
-  dmin=1000000;
-  for (i=0;i<colors;i++) {
-    d=sqr(r-Red[i])+sqr(g-Green[i])+sqr(b-Blue[i]);
-    if (d<dmin) {
-      dmin=d;
-      imin=i; } }
-  ppm_addtocolorhash(cht,&color,permi[imin]);
-  return imin;
-  }
-
-
-static int
-GetPixel( x, y )
-int x, y;
-    {
-    int color;
-
-    color = ppm_lookupcolor( cht, &pixels[y][x] );
-    if (color == -1)
-      color = closestcolor(pixels[y][x]);
-    else
-      color=perm[color];
-    return color;
+        presort_colorindex = ppm_lookupcolor(cmapP->cht, &pixels[y][x]);
+        if (presort_colorindex == -1)
+            presort_colorindex = 
+                closestcolor(pixels[y][x], input_maxval, cmapP);
+        colorindex = cmapP->perm[presort_colorindex];
     }
+    return colorindex;
+}
 
 
 #define TRUE 1
@@ -263,25 +276,33 @@ BumpPixel()
         }
 }
 
-/*
- * Return the next pixel from the image
- */
-static int
-GIFNextPixel( getpixel )
-ifunptr getpixel;
-{
-        int r;
 
-        if( CountDown == 0 )
-                return EOF;
 
-        --CountDown;
+static __inline__ int
+GIFNextPixel(pixel ** const pixels, pixval const input_maxval,
+             gray ** const alpha, gray const alpha_threshold, 
+             struct cmap * const cmapP) {
+/*----------------------------------------------------------------------------
+   Return the pre-sort color index (index into the unsorted GIF color map)
+   of the next pixel to be processed from the input image.
 
-        r = ( * getpixel )( curx, cury );
+   'alpha_threshold' is the gray level such that a pixel in the alpha
+   map whose value is less that that represents a transparent pixel
+   in the output.
+-----------------------------------------------------------------------------*/
+    int r;
 
-        BumpPixel();
-
-        return r;
+    if( CountDown == 0 )
+        return EOF;
+    
+    --CountDown;
+    
+    r = GetPixel(pixels, input_maxval, alpha, alpha_threshold, cmapP, 
+                 curx, cury);
+    
+    BumpPixel();
+    
+    return r;
 }
 
 
@@ -331,176 +352,6 @@ write_comment_extension(FILE *fp, const char comment[]) {
 
 
 
-/* public */
-
-static void
-GIFEncode(FILE *fp, const int GWidth, const int GHeight, const int GInterlace,
-          const int Background, const int Transparent,
-          const int BitsPerPixel, 
-          const int Red[], const int Green[], const int Blue[], 
-          ifunptr GetPixel, const char comment[]) {
-
-        int B;
-        int RWidth, RHeight;
-        int LeftOfs, TopOfs;
-        int Resolution;
-        int ColorMapSize;
-        int InitCodeSize;
-        int i;
-
-        Interlace = GInterlace;
-
-        ColorMapSize = 1 << BitsPerPixel;
-
-        RWidth = Width = GWidth;
-        RHeight = Height = GHeight;
-        LeftOfs = TopOfs = 0;
-
-        Resolution = BitsPerPixel;
-
-        /*
-         * Calculate number of bits we are expecting
-         */
-        CountDown = (long)Width * (long)Height;
-
-        /*
-         * Indicate which pass we are on (if interlace)
-         */
-        Pass = 0;
-
-        /*
-         * The initial code size
-         */
-        if( BitsPerPixel <= 1 )
-                InitCodeSize = 2;
-        else
-                InitCodeSize = BitsPerPixel;
-
-        /*
-         * Set up the current x and y position
-         */
-        curx = cury = 0;
-
-        /*
-         * Write the Magic header
-         */
-        if (Transparent != -1 || comment)
-            fwrite("GIF89a", 1, 6, fp);
-        else
-            fwrite("GIF87a", 1, 6, fp);
-
-        /*
-         * Write out the screen width and height
-         */
-        Putword( RWidth, fp );
-        Putword( RHeight, fp );
-
-        /*
-         * Indicate that there is a global colour map
-         */
-        B = 0x80;       /* Yes, there is a color map */
-
-        /*
-         * OR in the resolution
-         */
-        B |= (Resolution - 1) << 5;
-
-        /*
-         * OR in the Bits per Pixel
-         */
-        B |= (BitsPerPixel - 1);
-
-        /*
-         * Write it out
-         */
-        fputc( B, fp );
-
-        /*
-         * Write out the Background colour
-         */
-        fputc( Background, fp );
-
-        /*
-         * Byte of 0's (future expansion)
-         */
-        fputc( 0, fp );
-
-        /*
-         * Write out the Global Colour Map
-         */
-        for( i=0; i<ColorMapSize; ++i ) {
-                fputc( Red[i], fp );
-                fputc( Green[i], fp );
-                fputc( Blue[i], fp );
-        }
-
-        if ( Transparent >= 0 ) 
-            write_transparent_color_index_extension(fp, Transparent);
-
-        if ( comment )
-            write_comment_extension(fp, comment);
-
-        /*
-         * Write an Image separator
-         */
-        fputc( ',', fp );
-
-        /*
-         * Write the Image header
-         */
-
-        Putword( LeftOfs, fp );
-        Putword( TopOfs, fp );
-        Putword( Width, fp );
-        Putword( Height, fp );
-
-        /*
-         * Write out whether or not the image is interlaced
-         */
-        if( Interlace )
-                fputc( 0x40, fp );
-        else
-                fputc( 0x00, fp );
-
-        /*
-         * Write out the initial code size
-         */
-        fputc( InitCodeSize, fp );
-
-        /*
-         * Go and actually compress the data
-         */
-        compress( InitCodeSize+1, fp, GetPixel );
-
-        /*
-         * Write out a Zero-length packet (to end the series)
-         */
-        fputc( 0, fp );
-
-        /*
-         * Write the GIF file terminator
-         */
-        fputc( ';', fp );
-
-        /*
-         * And close the file
-         */
-        fclose( fp );
-}
-
-/*
- * Write out a word to the GIF file
- */
-static void
-Putword( w, fp )
-int w;
-FILE* fp;
-{
-        fputc( w & 0xff, fp );
-        fputc( (w / 256) & 0xff, fp );
-}
-
-
 /***************************************************************************
  *
  *  GIFCOMPR.C       - GIF Image compression routines
@@ -607,19 +458,24 @@ static int ClearCode;
 static int EOFCode;
 
 static void
-compress( init_bits, outfile, ReadValue )
-int init_bits;
-FILE* outfile;
-ifunptr ReadValue;
-{
-    register long fcode;
-    register code_int i /* = 0 */;
-    register int c;
-    register code_int ent;
-    register code_int disp;
-    register code_int hsize_reg;
-    register int hshift;
+compress(pixel ** const pixels, pixval const input_maxval,
+         gray ** const alpha, gray const alpha_maxval, 
+         struct cmap * const cmapP, 
+         int const init_bits, FILE * const outfile) {
 
+    long fcode;
+    code_int i /* = 0 */;
+    int c;
+    code_int ent;
+    code_int disp;
+    code_int hsize_reg;
+    int hshift;
+    bool eof;
+    gray const alpha_threshold = (alpha_maxval + 1) / 2;
+        /* gray levels below this in the alpha mask indicate transparent
+           pixels in the output image.
+        */
+    
     /*
      * Set up the globals:  g_init_bits - initial number of bits
      *                      g_outfile   - pointer to output file
@@ -642,7 +498,7 @@ ifunptr ReadValue;
 
     char_init();
 
-    ent = GIFNextPixel( ReadValue );
+    ent = GIFNextPixel(pixels, input_maxval, alpha, alpha_threshold, cmapP);
 
     hshift = 0;
     for ( fcode = (long) hsize;  fcode < 65536L; fcode *= 2L )
@@ -654,48 +510,44 @@ ifunptr ReadValue;
 
     output( (code_int)ClearCode );
 
-#ifdef SIGNED_COMPARE_SLOW
-    while ( (c = GIFNextPixel( ReadValue )) != (unsigned) EOF ) {
-#else /*SIGNED_COMPARE_SLOW*/
-    while ( (c = GIFNextPixel( ReadValue )) != EOF ) {  /* } */
-#endif /*SIGNED_COMPARE_SLOW*/
+    eof = FALSE;
+    while (!eof) {
+        c = GIFNextPixel(pixels, input_maxval, alpha, alpha_threshold, cmapP);
+        if (c == EOF) eof = TRUE;
+        if (!eof) {
+            ++in_count;
 
-        ++in_count;
+            fcode = (long) (((long) c << maxbits) + ent);
+            i = (((code_int)c << hshift) ^ ent);    /* xor hashing */
 
-        fcode = (long) (((long) c << maxbits) + ent);
-        i = (((code_int)c << hshift) ^ ent);    /* xor hashing */
+            if ( HashTabOf (i) == fcode ) {
+                ent = CodeTabOf (i);
+                continue;
+            } else if ( (long)HashTabOf (i) < 0 )      /* empty slot */
+                goto nomatch;
+            disp = hsize_reg - i;        /* secondary hash (after G. Knott) */
+            if ( i == 0 )
+                disp = 1;
+        probe:
+            if ( (i -= disp) < 0 )
+                i += hsize_reg;
 
-        if ( HashTabOf (i) == fcode ) {
-            ent = CodeTabOf (i);
-            continue;
-        } else if ( (long)HashTabOf (i) < 0 )      /* empty slot */
-            goto nomatch;
-        disp = hsize_reg - i;           /* secondary hash (after G. Knott) */
-        if ( i == 0 )
-            disp = 1;
-probe:
-        if ( (i -= disp) < 0 )
-            i += hsize_reg;
-
-        if ( HashTabOf (i) == fcode ) {
-            ent = CodeTabOf (i);
-            continue;
-        }
-        if ( (long)HashTabOf (i) > 0 )
-            goto probe;
-nomatch:
-        output ( (code_int) ent );
-        ++out_count;
-        ent = c;
-#ifdef SIGNED_COMPARE_SLOW
-        if ( (unsigned) free_ent < (unsigned) maxmaxcode) {
-#else /*SIGNED_COMPARE_SLOW*/
-        if ( free_ent < maxmaxcode ) {  /* } */
-#endif /*SIGNED_COMPARE_SLOW*/
-            CodeTabOf (i) = free_ent++; /* code -> hashtable */
-            HashTabOf (i) = fcode;
-        } else
+            if ( HashTabOf (i) == fcode ) {
+                ent = CodeTabOf (i);
+                continue;
+            }
+            if ( (long)HashTabOf (i) > 0 )
+                goto probe;
+        nomatch:
+            output ( (code_int) ent );
+            ++out_count;
+            ent = c;
+            if ( free_ent < maxmaxcode ) {  /* } */
+                CodeTabOf (i) = free_ent++; /* code -> hashtable */
+                HashTabOf (i) = fcode;
+            } else
                 cl_block();
+        }
     }
     /*
      * Put out the final code.
@@ -897,19 +749,396 @@ flush_char()
 
 
 
+static void
+GIFEncode(FILE * const fp, 
+          pixel ** const pixels, pixval const input_maxval,
+          int const GWidth, int const GHeight, 
+          gray ** const alpha, gray const alpha_maxval,
+          int const GInterlace, int const Background, 
+          int const BitsPerPixel, struct cmap * const cmapP,
+          const char comment[]) {
+
+    int B;
+    int RWidth, RHeight;
+    int LeftOfs, TopOfs;
+    int Resolution;
+    int ColorMapSize;
+    int InitCodeSize;
+    int i;
+
+    Interlace = GInterlace;
+
+    ColorMapSize = 1 << BitsPerPixel;
+
+    RWidth = Width = GWidth;
+    RHeight = Height = GHeight;
+    LeftOfs = TopOfs = 0;
+
+    Resolution = BitsPerPixel;
+
+    /* Calculate number of bits we are expecting */
+    CountDown = (long)Width * (long)Height;
+
+    /* Indicate which pass we are on (if interlace) */
+    Pass = 0;
+
+    /* The initial code size */
+    if( BitsPerPixel <= 1 )
+        InitCodeSize = 2;
+    else
+        InitCodeSize = BitsPerPixel;
+
+    /* Set up the current x and y position */
+    curx = cury = 0;
+
+    /* Write the Magic header */
+    if (cmapP->transparent != -1 || comment)
+        fwrite("GIF89a", 1, 6, fp);
+    else
+        fwrite("GIF87a", 1, 6, fp);
+
+    /* Write out the screen width and height */
+    Putword( RWidth, fp );
+    Putword( RHeight, fp );
+
+    /* Indicate that there is a global colour map */
+    B = 0x80;       /* Yes, there is a color map */
+
+    /* OR in the resolution */
+    B |= (Resolution - 1) << 5;
+
+    /* OR in the Bits per Pixel */
+    B |= (BitsPerPixel - 1);
+
+    /* Write it out */
+    fputc( B, fp );
+
+    /* Write out the Background color */
+    fputc( Background, fp );
+
+    /* Byte of 0's (future expansion) */
+    fputc( 0, fp );
+
+    /* Write out the Global Color Map */
+    for( i=0; i < ColorMapSize; ++i ) {
+        fputc( cmapP->red[i], fp );
+        fputc( cmapP->green[i], fp );
+        fputc( cmapP->blue[i], fp );
+    }
+
+    if ( cmapP->transparent >= 0 ) 
+        write_transparent_color_index_extension(fp, cmapP->transparent);
+
+    if ( comment )
+        write_comment_extension(fp, comment);
+
+    /* Write an Image separator */
+    fputc( ',', fp );
+
+    /* Write the Image header */
+
+    Putword( LeftOfs, fp );
+    Putword( TopOfs, fp );
+    Putword( Width, fp );
+    Putword( Height, fp );
+
+    /* Write out whether or not the image is interlaced */
+    if( Interlace )
+        fputc( 0x40, fp );
+    else
+        fputc( 0x00, fp );
+
+    /* Write out the initial code size */
+    fputc( InitCodeSize, fp );
+
+    /* Go and actually compress the data */
+    compress(pixels, input_maxval, alpha, alpha_maxval, cmapP, 
+             InitCodeSize+1, fp);
+
+    /* Write out a Zero-length packet (to end the series) */
+    fputc( 0, fp );
+
+    /* Write the GIF file terminator */
+    fputc( ';', fp );
+
+    /* And close the file */
+    fclose( fp );
+}
+
+/*
+ * Write out a word to the GIF file
+ */
+static void
+Putword( w, fp )
+int w;
+FILE* fp;
+{
+        fputc( w & 0xff, fp );
+        fputc( (w / 256) & 0xff, fp );
+}
+
+
+static int
+compute_transparent(const char colorarg[], 
+                    struct cmap * const cmapP) {
+/*----------------------------------------------------------------------------
+   Figure out the color index (index into the colormap) of the color
+   that is to be transparent in the GIF.
+
+   colorarg[] is the string that specifies the color the user wants to
+   be transparent (e.g. "red", "#fefefe").  Its maxval is the maxval
+   of the colormap.  'cmap' is the full colormap except that its
+   'transparent' component isn't valid.
+
+   colorarg[] is a standard Netpbm color specification, except that
+   may have a "=" prefix, which means it specifies a particular exact
+   color, as opposed to without the "=", which means "the color that
+   is closest to this and actually in the image."
+
+   Return -1 if colorarg[] specifies an exact color and that color is not
+   in the image.  Also issue an informational message.
+-----------------------------------------------------------------------------*/
+    int retval;
+
+    const char *colorspec;
+    bool exact;
+    int presort_colorindex;
+    pixel transcolor;
+
+    if (colorarg[0] == '=') {
+        colorspec = &colorarg[1];
+        exact = TRUE;
+    } else {
+        colorspec = colorarg;
+        exact = FALSE;
+    }
+        
+    transcolor = ppm_parsecolor((char*)colorspec, cmapP->maxval);
+    presort_colorindex = ppm_lookupcolor(cmapP->cht, &transcolor);
+    
+    if (presort_colorindex != -1)
+        retval = cmapP->perm[presort_colorindex];
+    else if (!exact)
+        retval = cmapP->perm[closestcolor(transcolor, cmapP->maxval, cmapP)];
+    else {
+        retval = -1;
+        pm_message(
+            "Warning: specified transparent color does not occur in image.");
+    }
+    return retval;
+}
+
+
+
+static void
+sort_colormap(int const sort, struct cmap * const cmapP) {
+/*----------------------------------------------------------------------------
+   Sort (in place) the colormap *cmapP.
+
+   Create the perm[] and permi[] mappings for the colormap.
+
+   'sort' is logical:  true means to sort the colormap by red intensity,
+   then by green intensity, then by blue intensity.  False means a null
+   sort -- leave it in the same order in which we found it.
+-----------------------------------------------------------------------------*/
+    int * const Red = cmapP->red;
+    int * const Blue = cmapP->blue;
+    int * const Green = cmapP->green;
+    int * const perm = cmapP->perm;
+    int * const permi = cmapP->permi;
+    unsigned int const cmapsize = cmapP->cmapsize;
+    
+    int i;
+
+    for (i=0; i < cmapsize; i++)
+        permi[i] = i;
+
+    if (sort) {
+        pm_message("sorting colormap");
+        for (i=0; i < cmapsize; i++) {
+            int j;
+            for (j=i+1; j < cmapsize; j++)
+                if (((Red[i]*MAXCMAPSIZE)+Green[i])*MAXCMAPSIZE+Blue[i] >
+                    ((Red[j]*MAXCMAPSIZE)+Green[j])*MAXCMAPSIZE+Blue[j]) {
+                    int tmp;
+                    
+                    tmp=permi[i]; permi[i]=permi[j]; permi[j]=tmp;
+                    tmp=Red[i]; Red[i]=Red[j]; Red[j]=tmp;
+                    tmp=Green[i]; Green[i]=Green[j]; Green[j]=tmp;
+                    tmp=Blue[i]; Blue[i]=Blue[j]; Blue[j]=tmp; } }
+    }
+
+    for (i=0; i < cmapsize; i++)
+        perm[permi[i]] = i;
+}
+
+
+
+static void
+normalize_to_255(colorhist_vector const chv, struct cmap * const cmapP) {
+/*----------------------------------------------------------------------------
+   With a PPM color histogram vector 'chv' as input, produce a colormap
+   of integers 0-255 as output in *cmapP.
+-----------------------------------------------------------------------------*/
+    int i;
+    pixval const maxval = cmapP->maxval;
+
+    if ( maxval != 255 )
+        pm_message(
+            "maxval is not 255 - automatically rescaling colors" );
+
+    for ( i = 0; i < cmapP->cmapsize; ++i ) {
+        if ( maxval == 255 ) {
+            cmapP->red[i] = (int) PPM_GETR( chv[i].color );
+            cmapP->green[i] = (int) PPM_GETG( chv[i].color );
+            cmapP->blue[i] = (int) PPM_GETB( chv[i].color );
+        } else {
+            cmapP->red[i] = (int) PPM_GETR( chv[i].color ) * 255 / maxval;
+            cmapP->green[i] = (int) PPM_GETG( chv[i].color ) * 255 / maxval;
+            cmapP->blue[i] = (int) PPM_GETB( chv[i].color ) * 255 / maxval;
+        }
+    }
+}
+
+
+
+static void add_to_colormap(struct cmap * const cmapP, char * const
+                            colorspec, int * new_indexP) {
+/*----------------------------------------------------------------------------
+  Add a new entry to the colormap.  Make the color that specified by
+  'colorspec', and return the index of the new entry as *new_indexP.
+
+  'colorspec' is a color specification given by the user, e.g.
+  "red" or "rgb:ff/03.0d".  The maxval for this color specification is
+  that for the colormap *cmapP.
+-----------------------------------------------------------------------------*/
+    pixel const transcolor = ppm_parsecolor((char*)colorspec, cmapP->maxval);
+    
+    *new_indexP = cmapP->cmapsize++; 
+
+    cmapP->red[*new_indexP] = PPM_GETR(transcolor);
+    cmapP->green[*new_indexP] = PPM_GETG(transcolor); 
+    cmapP->blue[*new_indexP] = PPM_GETB(transcolor); 
+}
+
+
+
+static void
+colormap_from_file(char filespec[], unsigned int const maxcolors,
+                   colorhist_vector * const chvP, pixval * const maxvalP,
+                   int * const colorsP) {
+/*----------------------------------------------------------------------------
+   Read a colormap from the PPM file filespec[].  Return the color histogram
+   vector (which is practically a colormap) of the input image as *cvhP
+   and the maxval for that histogram as *maxvalP.
+-----------------------------------------------------------------------------*/
+    FILE *mapfile;
+    int cols, rows;
+    pixel ** colormap_ppm;
+
+    mapfile = pm_openr(filespec);
+    colormap_ppm = ppm_readppm(mapfile, &cols, &rows, maxvalP);
+    pm_close(mapfile);
+    
+    /* Figure out the colormap from the <mapfile>. */
+    pm_message("computing other colormap...");
+    *chvP = 
+        ppm_computecolorhist(colormap_ppm, cols, rows, maxcolors, colorsP);
+    
+    ppm_freearray(colormap_ppm, rows); 
+}
+
+
+static void
+get_alpha(char * const alpha_filespec, int const cols, int const rows,
+          gray *** const alphaP, gray * const maxvalP) {
+
+    if (alpha_filespec) {
+        int alpha_cols, alpha_rows;
+        *alphaP = pgm_readpgm(pm_openr(alpha_filespec),
+                              &alpha_cols, &alpha_rows, maxvalP);
+        if (alpha_cols != cols || alpha_rows != rows)
+            pm_error("alpha mask is not the same dimensions as the "
+                     "input file (alpha is %dW x %dH; image is %dW x %dH)",
+                     alpha_cols, alpha_rows, cols, rows);
+    } else 
+        *alphaP = NULL;
+}
+
+
+static void
+compute_ppm_colormap(pixel ** const pixels, int const cols, int const rows,
+                     int const input_maxval, bool const have_alpha, 
+                     char * const mapfile, colorhist_vector * const chvP,
+                     colorhash_table * const chtP,
+                     pixval * const colormap_maxvalP, 
+                     int * const colorsP) {
+/*----------------------------------------------------------------------------
+   Compute a colormap, PPM style, for the image 'pixels', which is
+   'cols' by 'rows' with maxval 'input_maxval'.  If 'mapfile' is
+   non-null, Use the colors in that (PPM) file for the color map
+   instead of the colors in 'pixels'.
+
+   Return the colormap as *chvP and *chtP.  Return the maxval for that
+   colormap as *colormap_maxvalP.
+
+   While we're at it, count the colors and validate that there aren't
+   too many.  Return the count as *colorsP.  In determining if there are
+   too many, allow one slot for a fake transparency color if 'have_alpha'
+   is true.  If there are too many, issue an error message and abort the
+   program.
+-----------------------------------------------------------------------------*/
+    unsigned int maxcolors;
+        /* The most colors we can tolerate in the image.  If we have
+           our own made-up entry in the colormap for transparency, it
+           isn't included in this count.
+        */
+
+    if (have_alpha)
+        maxcolors = MAXCMAPSIZE - 1;
+    else
+        maxcolors = MAXCMAPSIZE;
+
+    if (mapfile) {
+        /* Read the colormap from a separate colormap file. */
+        colormap_from_file(mapfile, maxcolors, chvP, colormap_maxvalP, 
+                           colorsP);
+    } else {
+        /* Figure out the color map from the input file */
+        pm_message("computing colormap...");
+        *chvP = ppm_computecolorhist(pixels, cols, rows, maxcolors, colorsP); 
+        *colormap_maxvalP = input_maxval;
+    }
+
+    if (*chvP == NULL)
+        pm_error("too many colors - try doing a 'ppmquant %d'", maxcolors);
+    pm_message("%d colors found", *colorsP);
+
+    /* And make a hash table for fast lookup. */
+    *chtP = ppm_colorhisttocolorhash(*chvP, *colorsP);
+}
+
+
+
 int
 main(int argc, char *argv[]) {
     struct cmdline_info cmdline;
     FILE* ifp;
-    int rows, cols, i,j,k, BitsPerPixel;
-    pixel transcolor;
-    pixval colormap_maxval, input_maxval;
-    pixval maxval;
+    int rows, cols;
+    int BitsPerPixel;
+    pixel ** pixels;   /* The input image, in PPM format */
+    pixval input_maxval;  /* Maxval for 'pixels' */
+    gray ** alpha;     /* The supplied alpha mask; NULL if none */
+    gray alpha_maxval; /* Maxval for 'alpha' */
+
+    struct cmap cmap;
+        /* The colormap, with all its accessories */
     colorhist_vector chv;
-    int transparent;
-        /* color index number in GIF palette of the color that is to be
-           transparent.  -1 if no color is transparent 
-           */
+    int fake_transparent;
+        /* colormap index of the fake transparency color we're using to
+           implement the alpha mask.  Undefined if we're not doing an alpha
+           mask.
+        */
 
     ppm_init( &argc, argv );
 
@@ -921,98 +1150,47 @@ main(int argc, char *argv[]) {
 
     pm_close(ifp);
 
-    /* Figure out the colormap. */
+    get_alpha(cmdline.alpha_filespec, cols, rows, &alpha, &alpha_maxval);
 
-    if (cmdline.mapfile) {
-        /* Read the colormap from a separate colormap file. */
-        FILE *mapfile;
-        int cols, rows;
-
-        mapfile = pm_openr(cmdline.mapfile);
-        pixels = ppm_readppm(mapfile, &cols, &rows, &colormap_maxval);
-        pm_close(mapfile);
-
-        /* Figure out the colormap from the <mapfile>. */
-        pm_message("computing other colormap...");
-        chv = ppm_computecolorhist(pixels, cols, rows, MAXCOLORS, &colors);
-
-        ppm_freearray(pixels, rows); 
-    } else {
-        /* Figure out the color map from the input file */
-        pm_message("computing colormap...");
-        chv = ppm_computecolorhist(pixels, cols, rows, MAXCOLORS, &colors); 
-    }
-
-    maxval = (cmdline.mapfile) ? colormap_maxval : input_maxval;
-
-    if ( chv == (colorhist_vector) 0 )
-        pm_error(
-            "too many colors - try doing a 'ppmquant %d'", MAXCOLORS );
-    pm_message( "%d colors found", colors );
+    compute_ppm_colormap(pixels, cols, rows, input_maxval, (alpha != NULL), 
+                         cmdline.mapfile, 
+                         &chv, &cmap.cht, &cmap.maxval, &cmap.cmapsize);
 
     /* Now turn the ppm colormap into the appropriate GIF colormap. */
-    if ( maxval > 255 )
-        pm_message(
-            "maxval is not 255 - automatically rescaling colors" );
-    for ( i = 0; i < colors; ++i )
-        {
-        if ( maxval == 255 )
-            {
-            Red[i] = PPM_GETR( chv[i].color );
-            Green[i] = PPM_GETG( chv[i].color );
-            Blue[i] = PPM_GETB( chv[i].color );
-            }
-        else
-            {
-            Red[i] = (int) PPM_GETR( chv[i].color ) * 255 / maxval;
-            Green[i] = (int) PPM_GETG( chv[i].color ) * 255 / maxval;
-            Blue[i] = (int) PPM_GETB( chv[i].color ) * 255 / maxval;
-            }
-        }
 
-    /* Sort the colormap */
-    for (i=0;i<colors;i++)
-      permi[i]=i;
-    if (cmdline.sort) {
-      pm_message("sorting colormap");
-      for (i=0;i<colors;i++)
-        for (j=i+1;j<colors;j++)
-          if (((Red[i]*MAXCOLORS)+Green[i])*MAXCOLORS+Blue[i] >
-              ((Red[j]*MAXCOLORS)+Green[j])*MAXCOLORS+Blue[j]) {
-            k=permi[i]; permi[i]=permi[j]; permi[j]=k;
-            k=Red[i]; Red[i]=Red[j]; Red[j]=k;
-            k=Green[i]; Green[i]=Green[j]; Green[j]=k;
-            k=Blue[i]; Blue[i]=Blue[j]; Blue[j]=k; } }
-    for (i=0;i<colors;i++)
-      perm[permi[i]]=i;
+    normalize_to_255(chv, &cmap);
 
-    BitsPerPixel = colorstobpp( colors );
+    ppm_freecolorhist(chv);
 
-    /* And make a hash table for fast lookup. */
-    cht = ppm_colorhisttocolorhash( chv, colors );
-    ppm_freecolorhist( chv );
+    if (alpha) {
+        /* Add a fake entry to the end of the colormap for transparency.  
+           Make its color black. 
+        */
+        add_to_colormap(&cmap, cmdline.alphacolor, &fake_transparent);
+    }
+    sort_colormap(cmdline.sort, &cmap);
 
-    /* figure out the transparent colour index */
-    if (cmdline.transparent) {
-        int colorindex;
-
-        transcolor = ppm_parsecolor(cmdline.transparent, 255);
-    	colorindex = ppm_lookupcolor( cht, &transcolor );
-        
-        if (colorindex != -1)
-            transparent = perm[colorindex];
-        else
-            transparent = closestcolor(transcolor);
-    } else
-        transparent = -1;
+    BitsPerPixel = pm_maxvaltobits(cmap.cmapsize-1);
 
+    if (alpha) {
+        cmap.transparent = cmap.perm[fake_transparent];
+    } else {
+        if (cmdline.transparent)
+            cmap.transparent = 
+                compute_transparent(cmdline.transparent, &cmap);
+        else 
+            cmap.transparent = -1;
+    }
     /* All set, let's do it. */
-    GIFEncode(
-        stdout, cols, rows, cmdline.interlace, 0, transparent, BitsPerPixel,
-        Red, Green, Blue, GetPixel, cmdline.comment);
+    GIFEncode(stdout, pixels, input_maxval, cols, rows, 
+              alpha, alpha_maxval, 
+              cmdline.interlace, 0, BitsPerPixel, &cmap, cmdline.comment);
+
+    ppm_freearray(pixels, rows);
+    if (alpha)
+        pgm_freearray(alpha, rows);
 
     exit(0);
 }
 
 
-
